'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initializeFrom = exports.initialize = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _highland = require('highland');

var _highland2 = _interopRequireDefault(_highland);

var _faults = require('../faults');

var _utils = require('../utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _utils.debug)('pl:init');

let thePipelines = {};

const initialize = exports.initialize = (pipelines, opt) => {
  const keys = Object.keys(pipelines);

  debug('initialize: %j', keys);

  thePipelines = keys.reduce((accumulator, id) => _extends({}, accumulator, {
    [id]: pipelines[id](_extends({ // pass in options
      id
    }, opt, addDebug(id)))
  }), {});

  return { assemble: assemble(opt) };
};

const initializeFrom = exports.initializeFrom = rules => rules.reduce((accumulator, rule) => _extends({}, accumulator, {
  [rule.id]: opt => rule.flavor(_extends({}, rule, opt, rule) // include again for override precedence
  )
}), {});

const assemble = opt => (head, includeFaultHandler = true) => {
  const keys = Object.keys(thePipelines);

  debug('assemble: %j', keys);

  if (includeFaultHandler) {
    // after pre processoring
    head = head.errors(_faults.faults);
  }

  const lines = keys.map(key => {
    const f = thePipelines[key];
    const p = _highland2.default.pipeline(f);
    p.id = key;
    return p;
  });

  /* istanbul ignore else */
  if (lines.length > 0) {
    const last = lines.length - 1;

    lines.slice(0, last).forEach((p, i) => {
      debug('FORK: %s', p.id);
      const os = head.observe();

      lines[i] = os
      // shallow clone of data per pipeline
      .map(uow => _extends({
        pipeline: p.id
      }, uow, addDebug(p.id))).through(p);
    });

    debug('FORK: %s', lines[last].id);
    const p = lines[last];
    lines[last] = head.map(uow => _extends({
      pipeline: p.id
    }, uow, addDebug(p.id))).through(lines[last]);
  }

  let s = (0, _highland2.default)(lines).merge();

  if (includeFaultHandler) {
    s = s.errors(_faults.faults).through((0, _faults.flushFaults)(_extends({}, opt, addDebug('fault'))));
  }

  return s;
};

const addDebug = id => ({ debug: (0, _utils.debug)(`pl:${id}`) });