'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.publishToEventBridge = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _highland = require('highland');

var _highland2 = _interopRequireDefault(_highland);

var _eventbridge = require('../connectors/eventbridge');

var _eventbridge2 = _interopRequireDefault(_eventbridge);

var _batch = require('./batch');

var _faults = require('./faults');

var _print = require('./print');

var _tags = require('./tags');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const publishToEventBridge = exports.publishToEventBridge = ({ // eslint-disable-line import/prefer-default-export
  debug = (0, _print.debug)('eventbridge'),
  busName = process.env.BUS_NAME || 'undefined',
  source = 'custom',
  eventField = 'event',
  batchSize = Number(process.env.PUBLISH_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 25,
  parallel = Number(process.env.PUBLISH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  handleErrors = true
} = {}) => {
  const connector = new _eventbridge2.default({ debug });

  const toInputParams = batchUow => _extends({}, batchUow, {
    inputParams: {
      Entries: batchUow.batch.map(uow => ({
        EventBusName: busName,
        Source: source,
        DetailType: uow[eventField].type,
        Detail: JSON.stringify(uow[eventField])
      }))
    }
  });

  const putEvents = batchUow => {
    const p = connector.putEvents(batchUow.inputParams).catch((0, _faults.rejectWithFault)(batchUow, !handleErrors)).then(handleFailedEntries(batchUow)).then(publishResponse => _extends({}, batchUow, { publishResponse }));

    return (0, _highland2.default)(p); // wrap promise in a stream
  };

  return s => s.map((0, _tags.adornStandardTags)(eventField)).batch(batchSize).map(_batch.toBatchUow).map(toInputParams).map(putEvents).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};

const handleFailedEntries = batchUow => publishResponse => {
  if (publishResponse.FailedEntryCount === 0) {
    return publishResponse;
  } else {
    const failed = publishResponse.Entries.reduce((a, c, i) => {
      if (c.ErrorCode) {
        return [...a, _extends({}, batchUow.batch[i], {
          inputParam: batchUow.inputParams.Entries[i],
          err: {
            code: c.ErrorCode,
            msg: c.ErrorMessage
          }
        })];
      } else {
        return a;
      }
    }, []);

    return (0, _faults.throwFault)({ batch: failed })(new Error(`Event Bridge Failed Entry Count: ${publishResponse.FailedEntryCount}`));
  }
};