'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.toRecord = exports.publishToKinesis = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _highland = require('highland');

var _highland2 = _interopRequireDefault(_highland);

var _kinesis = require('../connectors/kinesis');

var _kinesis2 = _interopRequireDefault(_kinesis);

var _batch = require('./batch');

var _faults = require('./faults');

var _print = require('./print');

var _tags = require('./tags');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const publishToKinesis = exports.publishToKinesis = ({
  debug = (0, _print.debug)('kinesis'),
  streamName = process.env.STREAM_NAME,
  eventField = 'event',
  batchSize = Number(process.env.PUBLISH_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 25,
  parallel = Number(process.env.PUBLISH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  handleErrors = true
} = {}) => {
  const connector = new _kinesis2.default({ debug, streamName });

  const toInputParams = batchUow => _extends({}, batchUow, {
    inputParams: {
      Records: batchUow.batch.map(uow => toRecord(uow[eventField]))
    }
  });

  const putRecords = batchUow => {
    const p = connector.putRecords(batchUow.inputParams).then(publishResponse => _extends({}, batchUow, { publishResponse })).catch((0, _faults.rejectWithFault)(batchUow, !handleErrors));

    return (0, _highland2.default)(p); // wrap promise in a stream
  };

  return s => s.map((0, _tags.adornStandardTags)(eventField)).batch(batchSize).map(_batch.toBatchUow).map(toInputParams).map(putRecords).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};

const toRecord = exports.toRecord = e => ({
  Data: Buffer.from(JSON.stringify(e)),
  PartitionKey: e.partitionKey
});